@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-form-step-form-prefix-cls: ~'@{ant-prefix}-pro-steps-form';

.@{pro-form-step-form-prefix-cls} {
  &-container {
    width: max-content;
    min-width: 520px;
    max-width: 100%;
    margin: auto;
  }

  &-steps-container {
    max-width: 1160px;
    margin: auto;
  }

  &-step {
    display: none;
    margin-top: 32px;
    &-active {
      display: block;
    }

    > form {
      max-width: 100%;
    }
  }
}
