@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-table-prefix-cls: ~'@{ant-prefix}-pro-form';

// 只有在垂直模式隐藏label
.@{ant-prefix}-form:not(.@{ant-prefix}-form-horizontal) {
  .@{pro-table-prefix-cls}-list {
    &-item {
      .@{ant-prefix}-form-item-label {
        display: none;
      }
    }
  }
}

.@{pro-table-prefix-cls}-list {
  max-width: 100%;
  &-item {
    &&-show-label {
      .@{ant-prefix}-form-item-label {
        display: inline-block;
      }
    }
  }

  &-item {
    &:first-of-type {
      div:first-of-type {
        .@{ant-prefix}-form-item {
          .@{ant-prefix}-form-item-label {
            display: inline-block;
          }
        }
      }
    }
  }

  &-action {
    display: flex;
    height: 32px;
    margin-bottom: 24px;
    line-height: 32px;
  }

  .@{ant-prefix}-pro-card .@{ant-prefix}-pro-card-extra {
    .@{pro-table-prefix-cls}-list {
      &-action {
        margin-bottom: 0;
      }
    }
  }

  &-action-icon {
    margin-left: 8px;
    cursor: pointer;
    transition: color 0.3s ease-in-out;
    &:hover {
      color: @primary-color-hover;
    }
  }

  &-creator-button-top {
    margin-bottom: 24px;
  }
}
