@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';
@import '../../index';

.@{pro-table-prefix-cls}-toolbar {
  display: flex;
  align-items: center;
  justify-content: space-between;
  height: 64px;
  padding: 0 24px;

  &-option {
    display: flex;
    align-items: center;
    justify-content: flex-end;
  }

  &-title {
    flex: 1;
    color: @label-color;
    font-weight: 500;
    font-size: 16px;
    line-height: 24px;
    opacity: 0.85;
  }
}

@media (max-width: @screen-xs) {
  .@{pro-table-prefix-cls} {
    .ant-table {
      width: 100%;
      overflow-x: auto;
      &-thead > tr,
      &-tbody > tr {
        > th,
        > td {
          white-space: pre;
          > span {
            display: block;
          }
        }
      }
    }
  }
}

@media (max-width: 575px) {
  .@{pro-table-prefix-cls}-toolbar {
    flex-direction: column;
    align-items: flex-start;
    justify-content: flex-start;
    height: auto;
    margin-bottom: 16px;
    margin-left: 16px;
    padding: 8px;
    padding-top: 16px;
    line-height: normal;

    &-title {
      margin-bottom: 16px;
    }

    &-option {
      display: flex;
      justify-content: space-between;
      width: 100%;
    }

    &-default-option {
      display: flex;
      flex: 1;
      align-items: center;
      justify-content: flex-end;
    }
  }
}
