@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-list-toolbar: ~'@{ant-prefix}-pro-table-list-toolbar';

.@{pro-list-toolbar} {
  overflow-x: auto;
  line-height: 1;

  &-container {
    display: flex;
    justify-content: space-between;
    padding: @padding-md 0;
    &-mobile {
      flex-direction: column;
    }
  }

  &-title {
    display: flex;
    align-items: center;
    justify-content: flex-start;
    color: @heading-color;
    font-weight: 500;
    font-size: @font-size-lg;
  }

  &-search:not(:last-child) {
    display: flex;
    align-items: center;
    justify-content: flex-start;
  }

  &-setting-item {
    margin: 0 4px;
    color: @icon-color-hover;
    font-size: @font-size-lg;
    cursor: pointer;

    > span {
      display: block;
      width: 100%;
      height: 100%;
    }

    &:hover {
      color: @primary-color-hover;
    }
  }

  &-left {
    display: flex;
    align-items: center;
    justify-content: flex-start;
  }

  &-right {
    display: flex;
    justify-content: flex-end;
  }

  &-extra-line {
    margin-bottom: @margin-md;
  }

  &-filter {
    &:not(:last-child) {
      margin-right: @margin-md;
    }

    display: flex;
    align-items: center;

    .@{ant-prefix}-pro-table-search {
      margin: 0;
      padding: 0;
    }
  }

  &-inline-menu-item {
    display: inline-block;
    margin-right: @margin-lg;
    cursor: pointer;
    opacity: 0.75;

    &-active {
      font-weight: bold;
      opacity: 1;
    }
  }
  &-dropdownmenu-label {
    font-weight: bold;
    font-size: @font-size-lg;
    text-align: center;
    cursor: pointer;
  }

  .@{ant-prefix}-tabs-top > .@{ant-prefix}-tabs-nav {
    &::before {
      border-bottom: 0;
    }
    margin-bottom: 0;
    .@{ant-prefix}-tabs-nav-list {
      margin-top: 0;
      .@{ant-prefix}-tabs-tab {
        padding-top: 0;
      }
    }
  }
}

@media (max-width: 575px) {
  .@{pro-list-toolbar} {
    &-container {
      display: flex;
      flex-wrap: wrap;
    }
    &-left {
      margin-bottom: 16px;
    }
  }
}
