@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-table-alert-prefix-cls: ~'@{ant-prefix}-pro-table-alert';

.@{pro-table-alert-prefix-cls} {
  margin-bottom: 16px;
  // padding: 0 24px;

  .@{ant-prefix}-alert.@{ant-prefix}-alert-no-icon {
    padding: @padding-sm @padding-lg;
  }

  &-info {
    display: flex;
    align-items: center;
    transition: all 0.3s;
    &-content {
      flex: 1;
    }
    &-option {
      min-width: 48px;
      padding-left: 16px;
    }
  }
}
