@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-form-radio: ~'@{ant-prefix}-pro-field-radio';

.@{pro-form-radio}-vertical {
  .@{ant-prefix}-radio-wrapper {
    display: block;
    margin-right: 0;
  }
}
