@root-entry-name: 'default';
@import (reference) '~antd/es/style/themes/index.less';

@pro-field-date-picker: ~'@{ant-prefix}-pro-field-date-picker';

.@{pro-field-date-picker}-light {
  .@{ant-prefix}-calendar-picker,
  .@{ant-prefix}-picker {
    position: absolute;
    width: 80px;
    height: 28px;
    overflow: hidden;
    visibility: hidden;
  }
}
